# -*- coding: latin-1 -*-
'''
basePlugin.py

Copyright 2007 Mariano Nuez Di Croce @ CYBSEC

This file is part of sapyto, http://www.cybsec.com/EN/research/tools/sapyto.php

sapyto is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation version 2 of the License.

sapyto is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with sapyto; if not, write to the Free Software
Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
'''
from core.sapytoException import sapytoException

class basePlugin:

	def __init__( self ):
		self._conn = None
		
	def run( self ):
		"""
			This method is going to be implemented in every plugin.
		"""
		raise sapytoException('Plugin is not implementing required method run()' )
	
	def getName( self ):
		return self.__class__.__name__
		
	def setConn(self, conn):
		self._conn = conn
		
	def _checkConnection(self):
		if self._conn.is_connected():
			try:
				self._conn.close()
			except:
				raise sapytoException('Could not close connection.')
		try:
			self._conn.connect()
		except:
			raise sapytoException('Could not connect.')
		
		
	def getDesc( self ):
		'''
		@return: A description of the plugins functions.
		'''
		if self.__doc__ != None:
			res2 = self.__doc__.replace( '\t' , '' )
			res = ''.join ( [ i for i in res2.split('\n') if i != '' and i[0] != '@' ] )
		else:
			res = ''
		return res

	def getOptionsXML(self):
		'''
		This method returns a XML containing the Options that the plugin has.
		Using this XML the framework will build a window, a menu, or some other input method to retrieve
		the info from the user. The XML has to validate against the xml schema file located at :
		sapyto/core/ui/userInterface.dtd
		
		@return: XML with the plugin options.
		'''	
		return	''

	def setOptions( self, OptionList ):
		'''
		This method sets all the options that are configured using the user interface 
		generated by the framework using the result of getOptionsXML().
		
		@parameter OptionList: A dictionary with the options for the plugin.
		@return: No value is returned.
		'''	
		
	def setCommands(self, commands):
		''' Not implemented by BasePlugin plugins. Only implemented in baseShellPlugin plugins.
		'''
