# -*- coding: latin-1 -*-
'''
misc.py

Copyright 2007 Mariano Nuez Di Croce @ CYBSEC

This file is part of sapyto, http://www.cybsec.com/EN/research/tools/sapyto.php

sapyto is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation version 2 of the License.

sapyto is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with sapyto; if not, write to the Free Software
Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
'''
'''
This module defines some misc functions that are used around the project.

@author: Mariano Nuez Di Croce <mnunez@cybsec.com>
'''
import sys
import struct
import random
import string
import re

def factory(ModuleName, *args):
	'''
	This function creates an instance of a class thats inside a module
	with the same name.
		
	@parameter ModuleName: What do you want to instanciate ?
	@return: An instance.
	'''
	__import__(ModuleName)
	aModule = sys.modules[ModuleName]
	className = ModuleName.split('.')[len(ModuleName.split('.'))-1]
	aClass = getattr( aModule , className )
	return apply(aClass, args)

def fakeUnicode(raw):
	'''
	This functions receives a raw variable and peforms a conversion to Unicode.
	
	@parameter raw: Data to be converted
	@return: Unicode representation
	'''
	res = ''
	for i in raw:
		res += struct.pack('H', ord(i))
	return res

def reverseRFCPassword(passwd, fromConsole=True):
	'''
	This function de-obfuscates the RFC Password sent while performing RFC Function Calls.
	
	@parameter passwd: Obfuscated password
	@parameter fromConsole: Password read from console (needs mangling)
	@return: clear-text password
	'''
	tmp = ''
	if fromConsole:
		passwd = passwd.replace(' ', '')
		for i in range(0, len(passwd), 2):
			tmp = tmp + chr(string.atoi(passwd[i:i+2], 16))
		passwd = tmp
	
	key = [150, 222, 81, 30, 116, 14, 9, 9, 4, 27, 217, 70, 60, 53, 77, 142, 85, 197, 229, 212, 11, 160, 221, 214, 245, 33, 50, 15, 226, 205, 104, 79, 26, 80, 143, 117, 84, 134, 58, 187]
	res = ''
	i = 0
	for c in passwd:
		res += chr(key[i]^ord(c))
		i += 1
	return res

def mixCase(str):
	'''
		This function receives a string and returns the same string, but with mixed case.
		
		@parameter str: string to transform
		@return: mixed-case string
		
		@author: Mariano Nuez Di Croce <mnunez@cybsec.com>
	'''
	res = ''
	for c in str:
		if random.random()>0.5:
			res += string.upper(c)
		else:
			res += string.lower(c)
	return res
	
def getFS(opsys):
	'''
		This function gets the folder separator ('\' | '/') according to operating system.
		
		@parameter opsys: Operating system (as returned from sapinfo plugin)
		@returns: string containing folder separator
	'''
	fs = '/'
	if re.compile('Microsoft|Windows').search(opsys):
		fs = '\\'
	return fs
