# -*- coding: latin-1 -*-
'''
rfcServer.py

Copyright 2007 Mariano Nuez Di Croce @ CYBSEC

This file is part of sapyto, http://www.cybsec.com/EN/research/tools/sapyto.php

sapyto is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation version 2 of the License.

sapyto is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with sapyto; if not, write to the Free Software
Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
'''
import saprfc
import core.output.outputManager as om

class rfcServer:
	'''
	This class represent an external registered RFC Server.
	
	@author: Mariano Nuez Di Croce <mnunez@cybsec.com>
	'''
	
	def __init__(self, gwhost, gwserv, tpname, justForCallback=False):
		self._gwhost = gwhost
		self._gwserv =gwserv
		self._tpname = tpname
		self._conn = saprfc.conn(gwhost = self._gwhost, gwserv = self._gwserv, tpname = self._tpname)
		self._justForCallback = justForCallback
	
	def register(self):
		'''
		This method registers the RFC Server at the specified gateway
		'''
		try:
			return saprfc.conn.accept(self._conn, justForCallback = self._justForCallback)
		except:
			om.out.error('Registration of external RFC server \'' + self._tpname + '\' is not possible.')
	
	def addIface(self, iface, callback):
		'''
		This method adds an interface to the server.
		@parameter iface: Interface to addIface
		@parameter callback: callback handler
		'''
		iface.setCallback(callback)
		self._conn.iface(iface)
	
	def setGlobalHandler(self, callback):
		'''
		This method sets a Global Handler to trap any uninstalled functions. :P
		'''
		ifc = saprfc.iface("%%USER_GLOBAL_SERVER", callback=callback)
		#ifc.addParm( saprfc.parm("FOO", "", saprfc.RFCIMPORT, saprfc.RFCTYPE_CHAR, 30))
		self._conn.iface(ifc)
	
	def disconnect(self):
		self._conn.close()
	

