# -*- coding: latin-1 -*-
'''
pluginConfig.py

Copyright 2007 Mariano Nuez Di Croce @ CYBSEC

This file is part of sapyto, http://www.cybsec.com/EN/research/tools/sapyto.php

sapyto is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation version 2 of the License.

sapyto is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with sapyto; if not, write to the Free Software
Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
'''
# Import sapyto
import core.sapytoCore
import core.output.outputManager as om
from core.sapytoException import sapytoException
from core.ui.consoleUi.consoleMenu import consoleMenu

# for parsing plugin options
import xml.dom.minidom

class pluginConfig(consoleMenu):
	'''
	This class is a menu for configuring plugin internal options.
	
	@author: Andres Riancho ( andres.riancho@gmail.com )	
	'''
	def __init__( self, sapyto, commands=[] ):
		consoleMenu.__init__(self)
		self._menu = {'help':self._help, 'set':self._set,'view':self._view,'back':self._back}
		self._sapyto = sapyto
		self._commands = commands
		self._Options = {}
		self._pluginName = ''
	
	def sh( self, pluginName ):
		'''
		Starts the shell's main loop.
		
		@return: The prompt
		'''
		# First we need to get a plugin instance and set one or two parameters
		try:
			self._pluginInstance = self._sapyto.getPluginInstance( pluginName )
		except sapytoException, e:
			om.out.console('Error: ' + str(e) )
		else:
			self._parseXML( self._pluginInstance.getOptionsXML() )
			self._pluginName = pluginName
			
			prompt = 'sapyto/plugin/' + pluginName + '>>> '
			self._mainloop( prompt )

		
	def _exec( self, command ):
		'''
		Executes a user input.
		'''
		command, parameters = self._parse( command )
		
		if command in self._menu.keys():
			func = self._menu[command]
			return func(parameters)
		else:
			om.out.console( 'command not found' )
			return True

	def _parseXML( self, xmlString ):
		'''
		Parses the XML Options from a plugin.
		'''
		xmlString.replace( '\t' , '' )
		xmlDoc = xml.dom.minidom.parseString( xmlString )
		
		def setParameter( optionName, tag ):
			try:
				self._Options[ optionName ][ tag ] = option.getElementsByTagName(tag)[0].childNodes[0].data
			except:
				self._Options[ optionName ][ tag ] = ''
				
		for option in xmlDoc.getElementsByTagName('Option'):
			# I'm inside an option tag
			'''
			<Option name="checkPersistent">
				<default>True</default>
				<desc>Search persistent XSS</desc>
				<type>Boolean</type>
			</Option>
			'''
			optionName = option.getAttribute('name')
			self._Options[ optionName ] = {}
			
			setParameter( optionName, 'default' )
			setParameter( optionName, 'desc' )
			setParameter( optionName, 'help' )
			setParameter( optionName, 'type' )
			
		xmlDoc.unlink()
	
	def _help( self, parameters ):
		'''
		Prints a help message to the user.
		'''
		if len( parameters ) == 0:
			om.out.console('The following commands are available:')
			self.mprint('help','You are here. help [command] prints more specific help.')
			self.mprint('set' ,'Set a parameter value.')
			self.mprint('view','List all configuration parameters and current values.')
			self.mprint('back',	'Return to previous menu.')			

		else:
			if len( parameters ) == 1:
				if parameters[0] in self._menu.keys():
					if parameters[0] == 'set':
						om.out.console( 'Set a parameter value.')
						om.out.console( 'Sintax: set {parameter} {value}')
					elif parameters[0] == 'list':
						om.out.console( 'List all configuration parameters and current values.')
						om.out.console( 'Sintax: list')
		
		return True
	
	def _view( self , parameters):
		'''
		List all configurable parameters of a plugin.
		'''
		self.mprint('Parameter','Value')
		self.mprint('=========','=====')
		for value in self._Options.keys():
			self.mprint(value,self._Options[value]['default'])
		return True
	
	def _set( self , parameters):
		'''
		Configure a value.
		'''
		if len( parameters ) < 2:
			om.out.console('Invalid call to set, please see the help:')
			self._help( ['set'] )
		else:
			param = parameters[0]
			value = parameters[1:]
			value = ' '.join( value )
			if param in self._Options.keys():
				self._Options[ param ]['default'] = value
			else:
				om.out.console( 'Unknown parameter.' )
		return True
		
	def _back( self, parameters ):
		'''
		Calls plugin.setOptions(...) and then goes back.
		'''
		options4plugin = {}
		for value in self._Options.keys():
			# Only set the options that where changed
			if self._Options[value]['default'] != '':
				options4plugin[value] = self._Options[value]['default']
		### FIXME: This sends plugin options from output plugins to sapytoCore
		### and plugin options from [audit|attack] plugins to output manager
		### not really a problem... but it aint nice.
		self._sapyto.setPluginOptions( self._pluginName, options4plugin )
		om.out.setPluginOptions( self._pluginName, options4plugin )
		return False
