# -*- coding: latin-1 -*-
'''
plugins.py

Copyright 2007 Mariano Nuez Di Croce @ CYBSEC

This file is part of sapyto, http://www.cybsec.com/EN/research/tools/sapyto.php

sapyto is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation version 2 of the License.

sapyto is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with sapyto; if not, write to the Free Software
Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
'''
# Import sapyto
import core.sapytoCore
import core.output.outputManager as om
from core.sapytoException import sapytoException
from core.ui.consoleUi.pluginConfig import pluginConfig
from core.ui.consoleUi.consoleMenu import consoleMenu

class plugins(consoleMenu):
	'''
	This is the plugins configuration menu for the console.
	
	@author: Andres Riancho ( andres.riancho@gmail.com )	
	'''
	def __init__( self, sapyto , commands=[] ):
		consoleMenu.__init__(self)
		self._menu = {'help':self._help, 'list':self._list,'back':self._back}
		for i in sapyto.getPluginTypes():
			self._menu[ i ] = ''
		self._sapyto = sapyto
		self._commands = commands
		self._plugins = {}
		self._spaceSettings = 30
	
	def sh( self ):
		'''
		Starts the shell's main loop.
		
		@return: The prompt
		'''
		prompt = 'sapyto/plugins>>> '
		self._mainloop( prompt )		
		
	def _exec( self, command ):
		'''
		Executes a user input.
		'''
		command, parameters = self._parse( command )
		
		if command in self._sapyto.getPluginTypes():
			return self._pluginCfg( command, parameters )
		else:
			if command in self._menu.keys():
				func = self._menu[command]
				try:
					res = func(parameters)
				except sapytoException,e:
					om.out.console( str(e) )
				else:
					return res
			else:
				om.out.console( 'command not found' )
				return True
		
				
	def _help( self, parameters ):
		'''
		Prints a help message to the user.
		'''
		if len( parameters ) == 0:
			om.out.console('The following commands are available:')
			self.mprint('help','You are here. help [command] prints more specific help.')
			self.mprint('list','List all available plugins.')
			for p in self._sapyto.getPluginTypes():
				self.mprint( p ,'Enable and configure ' + p + ' plugins.')
			self.mprint('back','Return to previous menu.')

		else:
			if len( parameters ) == 1:
				if parameters[0] in self._menu.keys() or parameters[0] in self._sapyto.getPluginTypes():
					if parameters[0] == 'list':
						om.out.console( 'List all available plugins.')
						om.out.console( 'Sintax: list {plugin type}')
						om.out.console( 'Example: list audit')
					elif parameters[0] in self._sapyto.getPluginTypes():
						om.out.console( 'Enable and configure '+ parameters[0] +' plugins.' )
						om.out.console( 'Sintax: '+parameters[0]+' [config plugin1] [plugin1,plugin2 ... pluginN]')
						om.out.console( 'Example: '+parameters[0] )
						om.out.console( 'Result: All enabled '+parameters[0] + ' plugins are listed.')
						om.out.console( 'Example2: '+parameters[0]+ ' plugin1,plugin2')
						om.out.console( 'Result: plugin1 and plugin2 are configured to run')
						om.out.console( 'Example3: '+parameters[0]+ ' config plugin1')
						om.out.console( 'Result: Enters to the plugin configuration menu.' )
						om.out.console( 'Example4: '+parameters[0]+ ' all,!plugin3')
						om.out.console( 'Result: All ' + parameters[0] + ' plugins are configured to run except plugin3.' )
				else:
					om.out.console('Help not found.')
					
		
		return True
	
	def _pluginCfg( self , type, parameters):
		'''
		Handles enabling, disabling and configuring plugins.
		'''
		if len( parameters ) == 0 and type in self._sapyto.getPluginTypes():
			
			if type in self._plugins.keys():
				om.out.console( 'Enabled ' +type +' plugins:' )
				for plugin in self._plugins[ type ]:
					om.out.console( plugin )
			else:
				# Fill plugins with sapytoCore data
				self._plugins[type] = self._sapyto.getEnabledPlugins(type)					
				if self._plugins[ type ]:
					for plugin in self._plugins[ type ]:
						om.out.console( plugin )
				else:
					om.out.console( 'No '+ type +' plugins configured' )
		else:	
			if parameters[0] == 'config':
				if len( parameters ) == 1:
					om.out.console('Invalid command, please see the help:')
					self._help([type,])
				else:
					try:
						pConf = pluginConfig( self._sapyto, self._commands )
						pConf.sh(parameters[1])
					except KeyboardInterrupt:
						om.out.console( '' )
			else:
				plugins = ''.join(parameters[0:]).split(',')
				plugins = [ p.replace(' ', '') for p in plugins ]
				
				# Reverses the plugin list, ordering them
				plugins.reverse()
				# This avoids duplicates in the list
				plugins = list( set( plugins ) )	# bleh !
				pList = self._sapyto.getPluginList(  type  )
				for p in plugins:
					if p not in pList \
						and p.replace('!','') not in pList\
						and p != 'all':
							om.out.console('Unknown plugin selected.')
							return True
				self._plugins[ type ] = plugins
				self._sapyto.setPlugins( plugins, type )
				om.out.console( 'Enabled ' +type +' plugins:' )
				for plugin in self._plugins[ type ]:
					om.out.console( plugin )
		return True

	def _list(self , parameters):
		'''
		Lists all available [audit|attack|audit] plugins.
		'''
		if len( parameters ) == 0:
			om.out.console( 'To get a list of plugins use :' )
			om.out.console( 'Sintax: list {plugin type}' )
			om.out.console( 'Example: list audit' )
		else:
			if parameters[0] in self._sapyto.getPluginTypes():
				om.out.console( parameters[0] + ' plugins:' )
				list = self._sapyto.getPluginList( parameters[0] )
				for plugin in list:
					pluginInstance = self._sapyto.getPluginInstance( plugin )
					self.mprint( '\t' + plugin, pluginInstance.getDesc() )
		return True
		
	def _back( self, parameters ):
		# This is done here so output plugin options are applied.
		if 'output' not in self._plugins.keys():
			self._plugins['output'] = ['console']
		self._sapyto.setPlugins( self._plugins['output'] , 'output' )
		return False
