# -*- coding: latin-1 -*-
'''
stickShell.py

Copyright 2007 Mariano Nuez Di Croce @ CYBSEC

This file is part of sapyto, http://www.cybsec.com/EN/research/tools/sapyto.php

sapyto is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation version 2 of the License.

sapyto is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with sapyto; if not, write to the Free Software
Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
'''
# Import sapyto
import core.sapytoCore
import core.output.outputManager as om
from core.sapytoException import sapytoException
from core.ui.consoleUi.consoleMenu import consoleMenu

# for parsing targets options
import xml.dom.minidom

class stickShell(consoleMenu):
	'''
	This is the stickShell menu for the console.
	
	@author: Mariano Nuez Di Croce <mnunez@cybsec.com>
	'''
	def __init__( self, sapyto , commands=[] ):
		consoleMenu.__init__(self)
		self._menu = {'help':self._help, 'call':self._call, 'disconnect':self._disconnect, 'back':self._back}
		self._sapyto = sapyto
		self._commands = commands
		self._Options = {}
	
	def sh( self ):
		'''
		Starts the shell's main loop.
		
		@return: The prompt
		'''
		prompt = 'sapyto/attack/stickShell>>> '
		self._parseXML(self._getOptionsXML())
		self._mainloop( prompt )		
	
	def _parseXML( self, xmlString ):
		'''
		Parses the XML Options from a plugin.
		'''
		xmlString.replace( '\t' , '' )
		xmlDoc = xml.dom.minidom.parseString( xmlString )
		
		def setParameter( optionName, tag ):
			try:
				self._Options[ optionName ][ tag ] = option.getElementsByTagName(tag)[0].childNodes[0].data
			except:
				self._Options[ optionName ][ tag ] = ''
				
		for option in xmlDoc.getElementsByTagName('Option'):
			
			optionName = option.getAttribute('name')
			self._Options[ optionName ] = {}
			
			setParameter( optionName, 'default' )
			setParameter( optionName, 'desc' )
			setParameter( optionName, 'help' )
			setParameter( optionName, 'type' )
			
		xmlDoc.unlink()
	
	
	def _exec( self, command ):
		'''
		Executes a user input.
		'''
		command, parameters = self._parse( command )
		
		if command in self._menu.keys():
			func = self._menu[command]
			try:
				res = func(parameters)
			except sapytoException,e:
				om.out.console( str(e) )
			else:
				return res
		else:
			om.out.console( 'command not found' )
		return True
		
	def _call( self , parameters):
		'''
		Configure a value.
		'''
		om.out.console('Not implemented.')
		return True

	def _help( self, parameters ):
		'''
		Prints a help message to the user.
		'''
		if len(parameters) == 0:
			om.out.console('The following commands are available:')
			self.mprint('help','You are here.')
			self.mprint('call','Perform a raw RFC Call')
			self.mprint('disconnect','Disconnect remote RFC Server')
			self.mprint('back','Return to previous menu.')
		else:
			if len( parameters ) == 1:
				if parameters[0] in self._menu.keys():
					# Add commands help
					pass
		return True
	
	def _disconnect( self , parameters):
		'''
		Disconnects from current RFC Server
		'''
		conn = self._sapyto.getConn()
		conn.close()
		return True
		
	def _back( self, parameters ):
		return False

	def _getOptionsXML(self):
		return '<?xml version="1.0" encoding="ISO-8859-1"?>\
		<OptionList>\
		</OptionList>\
		'
