# -*- coding: latin-1 -*-
'''
targets.py

Copyright 2007 Mariano Nuez Di Croce @ CYBSEC

This file is part of sapyto, http://www.cybsec.com/EN/research/tools/sapyto.php

sapyto is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation version 2 of the License.

sapyto is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with sapyto; if not, write to the Free Software
Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
'''
# Import sapyto
import core.sapytoCore
import core.output.outputManager as om
from core.sapytoException import sapytoException
from core.ui.consoleUi.pluginConfig import pluginConfig
from core.ui.consoleUi.consoleMenu import consoleMenu

# for parsing targets options
import xml.dom.minidom

class targets(consoleMenu):
	'''
	This is the target configuration menu for the console.
	
	@author: Mariano Nuez Di Croce <mnunez@cybsec.com>
	'''
	def __init__( self, sapyto , commands=[] ):
		consoleMenu.__init__(self)
		self._menu = {'help':self._help, 'view':self._view, 'set':self._set, 'back':self._back}
		self._sapyto = sapyto
		self._commands = commands
		self._Options = {}
	
	def sh( self ):
		'''
		Starts the shell's main loop.
		
		@return: The prompt
		'''
		prompt = 'sapyto/targets>>> '
		self._parseXML(self._getOptionsXML())
		self._mainloop( prompt )		
	
	def _parseXML( self, xmlString ):
		'''
		Parses the XML Options from a plugin.
		'''
		xmlString.replace( '\t' , '' )
		xmlDoc = xml.dom.minidom.parseString( xmlString )
		
		def setParameter( optionName, tag ):
			try:
				self._Options[ optionName ][ tag ] = option.getElementsByTagName(tag)[0].childNodes[0].data
			except:
				self._Options[ optionName ][ tag ] = ''
				
		for option in xmlDoc.getElementsByTagName('Option'):
			
			optionName = option.getAttribute('name')
			self._Options[ optionName ] = {}
			
			setParameter( optionName, 'default' )
			setParameter( optionName, 'desc' )
			setParameter( optionName, 'help' )
			setParameter( optionName, 'type' )
			
		xmlDoc.unlink()
	
	
	def _exec( self, command ):
		'''
		Executes a user input.
		'''
		command, parameters = self._parse( command )
		
		if command in self._menu.keys():
			func = self._menu[command]
			try:
				res = func(parameters)
			except sapytoException,e:
				om.out.console( str(e) )
			else:
				return res
		else:
			om.out.console( 'command not found' )
		return True
		
	def _set( self , parameters):
		'''
		Configure a value.
		'''
		if len( parameters ) < 2:
			om.out.console('Invalid call to set, please see the help:')
			self._help( ['set'] )
		else:
			param = parameters[0]
			value = parameters[1:]
			value = ' '.join( value )
			if param in self._Options.keys():
				self._Options[ param ]['default'] = value
			else:
				om.out.console( 'Unknown parameter.' )
		return True

	def _help( self, parameters ):
		'''
		Prints a help message to the user.
		'''
		if len(parameters) == 0:
			om.out.console('The following commands are available:')
			self.mprint('help','You are here.')
			self.mprint('view','View defined target.')
			self.mprint('set','Set target information.')
			self.mprint('back','Return to previous menu.')
		else:
			if len( parameters ) == 1:
				if parameters[0] in self._menu.keys():
					if parameters[0] == 'set':
						om.out.console( 'Set target information:')
						self.mprint( 'type','Connection type: SAP|EXT')
						self.mprint( 'ashost','SAP R/3 Application Server host [e.g: 127.0.0.1]')
						self.mprint( 'sysnr','System Number [e.g: 00]')
						self.mprint( 'client','SAP Client Number [e.g: 000]')
						self.mprint( 'user','Username for connection [e.g: SAP*]')
						self.mprint( 'passwd','Username password [e.g: sapstar]')
						self.mprint( 'gwhost','Gateway host [e.g: 127.0.0.1]')
						self.mprint( 'gwserv','Gateway service [e.g: sapgw00]')
						self.mprint( 'tpname','External server tpname [e.g: extserv1]')
		return True
	
	def _view( self , parameters):
		'''
		List all configurable parameters of a plugin.
		'''
		self.mprint('Parameter','Value')
		self.mprint('=========','=====')
		for value in self._Options.keys():
			self.mprint(value,self._Options[value]['default'])
		return True
		
	def _back( self, parameters ):
		# Save target information
		self._sapyto.setTarget(self._Options)
		return False

	def _getOptionsXML(self):
		return '<?xml version="1.0" encoding="ISO-8859-1"?>\
		<OptionList>\
			<Option name="type">\
				<default>SAP</default>\
				<desc>Connection Type</desc>\
				<type>String</type>\
			</Option>\
			<Option name="ashost">\
				<default>127.0.0.1</default>\
				<desc>SAP R/3 Application server host</desc>\
				<type>Boolean</type>\
			</Option>\
			<Option name="sysnr">\
				<default>00</default>\
				<desc>System Number</desc>\
				<type>String</type>\
			</Option>\
			<Option name="client">\
				<default>000</default>\
				<desc>Client</desc>\
				<type>String</type>\
			</Option>\
			<Option name="user">\
				<default>SAP*</default>\
				<desc>Username</desc>\
				<type>String</type>\
			</Option>\
			<Option name="passwd">\
				<default>sapstar</default>\
				<desc>Username password</desc>\
				<type>String</type>\
			</Option>\
			<Option name="gwhost">\
				<default></default>\
				<desc>Gateway host</desc>\
				<type>String</type>\
			</Option>\
			<Option name="gwserv">\
				<default>sapgw00</default>\
				<desc>Gateway service</desc>\
				<type>String</type>\
			</Option>\
			<Option name="tpname">\
				<default></default>\
				<desc>External server tpname</desc>\
				<type>String</type>\
			</Option>\
		</OptionList>\
		'
