# -*- coding: latin-1 -*-
'''
stick.py

Copyright 2007 Mariano Nuez Di Croce @ CYBSEC

This file is part of sapyto, http://www.cybsec.com/EN/research/tools/sapyto.php

sapyto is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation version 2 of the License.

sapyto is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with sapyto; if not, write to the Free Software
Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
'''
from core.basePlugins.baseShellPlugin import baseShellPlugin
import core.kb.knowledgeBase as kb
import core.output.outputManager as om
import saprfcutil
from saprfc import *
from core.misc import *
import core.sapytoCore

import xml.dom.minidom

class stick(baseShellPlugin):
	'''
	Sticks to the RFC Server. Usefull for raw requests, blocking connections, etc.
	
	@author: Mariano Nuez Di Croce <mnunez@cybsec.com>
	'''
	def __init__( self):
		baseShellPlugin.__init__(self)
		self._menu = {'help':self._help, 'connect':self._connect, 'disconnect':self._disconnect, 'call':self._call, 'back':self._back}
		self._Options = {}
	
	def sh( self ):
		'''
		Starts the shell's main loop.
		
		@return: The prompt
		'''
		prompt = 'sapyto/attack/stickShell>>> '
		self._parseXML(self._getOptionsXML())
		self._mainloop( prompt )		
	
	def _parseXML( self, xmlString ):
		'''
		Parses the XML Options from a plugin.
		'''
		xmlString.replace( '\t' , '' )
		xmlDoc = xml.dom.minidom.parseString( xmlString )
		
		def setParameter( optionName, tag ):
			try:
				self._Options[ optionName ][ tag ] = option.getElementsByTagName(tag)[0].childNodes[0].data
			except:
				self._Options[ optionName ][ tag ] = ''
				
		for option in xmlDoc.getElementsByTagName('Option'):
			
			optionName = option.getAttribute('name')
			self._Options[ optionName ] = {}
			
			setParameter( optionName, 'default' )
			setParameter( optionName, 'desc' )
			setParameter( optionName, 'help' )
			setParameter( optionName, 'type' )
			
		xmlDoc.unlink()
	
	
	def _exec( self, command ):
		'''
		Executes a user input.
		'''
		command, parameters = self._parse( command )
		
		if command in self._menu.keys():
			func = self._menu[command]
			try:
				res = func(parameters)
			except sapytoException,e:
				om.out.console( str(e) )
			else:
				return res
		else:
			om.out.console( 'command not found' )
		return True
		
	
	def _help( self, parameters ):
		'''
		Prints a help message to the user.
		'''
		om.out.console('The following commands are available:')
		self.mprint('help','You are here.')
		self.mprint('connect','Connect to target.')
		self.mprint('call','Perform a raw RFC call.')
		self.mprint('disconnect','Close connection with RFC server.')
		self.mprint('back','Return to previous menu.')
		return True
	
	def _back( self, parameters ):
		return False

	def _getOptionsXML(self):
		return '<?xml version="1.0" encoding="ISO-8859-1"?>\
		<OptionList>\
		</OptionList>\
		'

	def run( self ):
		# TODO: Validate Targets and use of sapinfo
		om.out.debug('Inside stick plugin...')
		# Run stickShell
		self.sh()
	
	def _connect(self, parameters):
		om.out.information('Trying to connect...')
		self._checkConnection()
		om.out.information('Sticked to remote RFC Server')
		return True
	
	def _disconnect(self, parameters):
		self._conn.close()
		om.out.information('Disconnected.')
		return True

	def _call(self, parameters):
		om.out.information('Not implemented.')
		return True
