# -*- coding: latin-1 -*-
'''
checkRfcexec.py

Copyright 2007 Mariano Nuez Di Croce @ CYBSEC

This file is part of sapyto, http://www.cybsec.com/EN/research/tools/sapyto.php

sapyto is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation version 2 of the License.

sapyto is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with sapyto; if not, write to the Free Software
Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
'''
import core.sapytoCore as core
from core.basePlugins.basePlugin import basePlugin
import core.kb.knowledgeBase as kb
import core.output.outputManager as om
import saprfcutil
from saprfc import *

class checkRfcexec(basePlugin):
	'''
	This plugin detects if rfcexec is installed in a SAP Application Server. 
	
	@author: Mariano Nuez Di Croce <mnunez@cybsec.com>
	'''
	def __init__( self):
		basePlugin.__init__( self )
		self._iface = None
		self._targets = {'RFCSYSID':['^\w{3}$']}
		self._gwhost = 'Current target'
		self._gwserv = 'sapgw00'		
		
	def run( self ):
		om.out.information('Starting \'checkRfcexec\' audit plugin.')
		om.out.information('Checking Target Information: ', newLine=False)
		if core.sCore.validateTargets(self._targets):
			om.out.information('OK')
		else:
			om.out.information('Not in plugin \'targets\' list. Plugin may fail.')
		
		if self._gwhost == 'Current target':
			self._serverName = self._conn.ashost
		else:
			self._serverName = self._gwhost
		
		# We try different combinations of Program IDs.
		tpnames = ['rfcexec', 'execute', 'exec', 'run']
		tps = tpnames[:]
		for tp in tpnames:
			tps.append(self._serverName + '.' + tp)
		for tp in tps:
			om.out.information('\tTrying tpname: ' + tp)
			self._conn = conn(gwhost = self._serverName, gwserv= self._gwserv, type = 'E', tpname = tp)
			try:
				self._conn.connect()
			except:
				pass
			else:
				try:
					# Ping to keep rfcexec alive
					self._iface = iface("RFC_PING")
					if not saprfcutil.call_receive( self._conn, self._iface ):
						raise sapytoException('Could not perform RFC call.')
				except TypeError, e:
					self._conn.close()
				else:
					om.out.information('\n\tBingo! Found tpname: ' + tp)
					kb.kb.append('checkRfcexec', 'tpname', tp)
					kb.kb.save('checkRfcexec', 'called', 1)
					om.out.information('Plugin \'checkRfcexec\' finished.')
					self._conn.close()
					return
		om.out.information('\n\trfcexec was not found.')
		kb.kb.save('checkRfcexec', 'called', 1)
		om.out.information('Plugin \'checkRfcexec\' finished.')

	def getOptionsXML(self):
		'''
		This method returns a XML containing the Options that the plugin has.
		Using this XML the framework will build a window, a menu, or some other input method to retrieve
		the info from the user. The XML has to validate against the xml schema file located at :
		sapyto/core/ui/userInterface.dtd
		
		@return: XML with the plugin options.
		'''	
		return	'<?xml version="1.0" encoding="ISO-8859-1"?>\
		<OptionList>\
			<Option name="gwhost">\
				<default>Current target</default>\
				<desc>Gateway host. (use DNS name)</desc>\
				<type>String</type>\
			</Option>\
			<Option name="gwserv">\
				<default>sapgw00</default>\
				<desc>Gateway service</desc>\
				<type>String</type>\
			</Option>\
		</OptionList>\
		'

	def setGwhost(self, gwhost):
		self._gwhost = gwhost
	
	def setGwserv(self, gwserv):
		self._service = gwserv
	
	def setOptions( self, OptionList ):
		'''
		This method sets all the options that are configured using the user interface 
		generated by the framework using the result of getOptionsXML().
		
		@parameter OptionList: A dictionary with the options for the plugin.
		@return: No value is returned.
		'''	
		if 'gwhost' in OptionList.keys(): 
			self._gwhost = OptionList['gwhost']
		if 'gwserv' in OptionList.keys(): 
			self._gwserv = OptionList['gwserv']
