# -*- coding: latin-1 -*-
'''
ext_getDocu.py

Copyright 2007 Mariano Nuez Di Croce @ CYBSEC

This file is part of sapyto, http://www.cybsec.com/EN/research/tools/sapyto.php

sapyto is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation version 2 of the License.

sapyto is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with sapyto; if not, write to the Free Software
Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
'''
import core.sapytoCore as core
from core.basePlugins.basePlugin import basePlugin
import core.kb.knowledgeBase as kb
import core.output.outputManager as om
from saprfc import *
import saprfcutil

class ext_getDocu(basePlugin):
	'''
	Get Documentation of installed RFC functions on remote server.
	
	@author: Mariano Nuez Di Croce <mnunez@cybsec.com>
	'''
	def __init__( self):
		basePlugin.__init__( self )
		self._targets = {'RFCSYSID':['\(extern\)']}
		
	def run( self ):
		om.out.information('Starting \'ext_rfcDocu\' audit plugin.')
		om.out.information('Checking Target Information: ', newLine=False)
		if core.sCore.validateTargets(self._targets):
			om.out.information('OK')
		else:
			om.out.information('Not in plugin \'targets\' list. Plugin may fail.')
		
		self._iface = iface('RFC_DOCU')				
		t_docu = parm( 'DOCU', '', RFCTABLE, RFCTYPE_CHAR, 80)
		self._iface.addParm(t_docu)
		try:
			self._checkConnection()
		except sapytoException:
			raise
		try:
			if not saprfcutil.call_receive( self._conn, self._iface ):
				raise sapytoException('Could not perform RFC call.')
		except TypeError, e:
			self._conn.close()
		else:
			om.out.information('Retreiving system documentation: ')
			res = ''
			for i in  t_docu.value:
				res += '\t' + i + '\n'
			om.out.information(res)
			kb.kb.save('rfcDocu', 'documentation', res)
			self._conn.close()
			om.out.information('Plugin \'ext_rfcDocu\' finished.')

	def getOptionsXML(self):
		'''
		This method returns a XML containing the Options that the plugin has.
		Using this XML the framework will build a window, a menu, or some other input method to retrieve
		the info from the user. The XML has to validate against the xml schema file located at :
		sapyto/core/ui/userInterface.dtd
		
		@return: XML with the plugin options.
		'''	
		return	'<?xml version="1.0" encoding="ISO-8859-1"?>\
		<OptionList>\
		</OptionList>\
		'

	def setOptions( self, OptionList ):
		'''
		This method sets all the options that are configured using the user interface 
		generated by the framework using the result of getOptionsXML().
		
		@parameter OptionList: A dictionary with the options for the plugin.
		@return: No value is returned.
		'''	
