# -*- coding: latin-1 -*-
'''
ping.py

Copyright 2007 Mariano Nuez Di Croce @ CYBSEC

This file is part of sapyto, http://www.cybsec.com/EN/research/tools/sapyto.php

sapyto is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation version 2 of the License.

sapyto is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with sapyto; if not, write to the Free Software
Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
'''
from core.basePlugins.basePlugin import basePlugin
import core.output.outputManager as om
import core.kb.knowledgeBase as kb
from saprfc import *
import saprfcutil
import re

class ping(basePlugin):
	'''
	Check if remote RFC server is alive.
	
	@author: Mariano Nuez Di Croce <mnunez@cybsec.com>
	'''
	def __init__( self):
		basePlugin.__init__( self )
		self._iface = None
		self._targets = {}
		
	def run( self ):
		om.out.information('Starting \'ping\' audit plugin.')
		self._iface = iface('RFC_PING')
		status = False
		try:
			self._checkConnection()
		except sapytoException:
			om.out.information('System down.')
		else:
			try:
				if not saprfcutil.call_receive( self._conn, self._iface ):
					raise sapytoException('Could not perform RFC call.')
			except:
				om.out.information('System down.')
			else:
				om.out.information('System available.')
				self._conn.close()
				status = True
		om.out.information('Plugin \'ping\' finished.')
		return status
	
	def getOptionsXML(self):
		'''
		This method returns a XML containing the Options that the plugin has.
		Using this XML the framework will build a window, a menu, or some other input method to retrieve
		the info from the user. The XML has to validate against the xml schema file located at :
		sapyto/core/ui/userInterface.dtd
		
		@return: XML with the plugin options.
		'''	
		return	'<?xml version="1.0" encoding="ISO-8859-1"?>\
		<OptionList>\
		</OptionList>\
		'

	def setOptions( self, OptionList ):
		'''
		This method sets all the options that are configured using the user interface 
		generated by the framework using the result of getOptionsXML().
		
		@parameter OptionList: A dictionary with the options for the plugin.
		@return: No value is returned.
		'''	
		pass
