# -*- coding: latin-1 -*-
'''
registerExtServer.py

Copyright 2007 Mariano Nuez Di Croce @ CYBSEC

This file is part of sapyto, http://www.cybsec.com/EN/research/tools/sapyto.php

sapyto is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation version 2 of the License.

sapyto is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with sapyto; if not, write to the Free Software
Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
'''
import core.sapytoCore as core
from core.basePlugins.basePlugin import basePlugin
import core.kb.knowledgeBase as kb
import core.output.outputManager as om
from saprfc import *

class registerExtServer(basePlugin):
	'''
	Detects if it is possible to register an external RFC server.
	
	@author: Mariano Nuez Di Croce <mnunez@cybsec.com>
	'''
	def __init__( self):
		basePlugin.__init__( self )
		self._iface = None
		self._targets = {'RFCSYSID':['^\w{3}$']}
		self._gwhost = 'Current target'
		self._gwserv = 'sapgw00'		
		self._tpname = 'sapyto.sapyto'
		
	def run(self):
		om.out.information('Starting \'registerExtServer\' audit plugin.')
		om.out.information('Checking Target Information: ', newLine=False)
		if core.sCore.validateTargets(self._targets):
			om.out.information('OK')
		else:
			om.out.information('Not in plugin \'targets\' list. Plugin won\'t execute.')
			return
	
		if self._tpname == '':
			self._tpname = 'sapyto.sapyto'
		
		if self._gwhost == 'Current target':
			self._serverName = self._conn.ashost
		else:
			self._serverName = self._gwhost
		
		self._conn = conn(gwhost = self._serverName, gwserv = self._gwserv, tpname = self._tpname)
		
		try:
			res = self._conn.tryRegister() 
		except TypeError, e:
			kb.kb.save('registerExtServer', 'canRegisterExt', 0)
			om.out.information('Error: ' + str(e))
		else:
			kb.kb.save('registerExtServer', 'canRegisterExt', 1)
			om.out.information('External Server Registration is possible.')
		om.out.information('Plugin \'registerExtServer\' finished.')
		

	def getOptionsXML(self):
		'''
		This method returns a XML containing the Options that the plugin has.
		Using this XML the framework will build a window, a menu, or some other input method to retrieve
		the info from the user. The XML has to validate against the xml schema file located at :
		sapyto/core/ui/userInterface.dtd
		
		@return: XML with the plugin options.
		'''	
		return	'<?xml version="1.0" encoding="ISO-8859-1"?>\
		<OptionList>\
			<Option name="gwhost">\
				<default>Current target</default>\
				<desc>Gateway host. (use DNS name)</desc>\
				<type>String</type>\
			</Option>\
			<Option name="gwserv">\
				<default>sapgw00</default>\
				<desc>Gateway service</desc>\
				<type>String</type>\
			</Option>\
			<Option name="tpname">\
				<default>sapyto.sapyto</default>\
				<desc>TPname to register.</desc>\
				<type>String</type>\
			</Option>\
		</OptionList>\
		'
	
	def setOptions( self, OptionList ):
		'''
		This method sets all the options that are configured using the user interface 
		generated by the framework using the result of getOptionsXML().
		
		@parameter OptionList: A dictionary with the options for the plugin.
		@return: No value is returned.
		'''	
		if 'gwhost' in OptionList.keys(): 
			self._gwhost = OptionList['gwhost']
		if 'gwserv' in OptionList.keys(): 
			self._gwserv = OptionList['gwserv']
		if 'tpname' in OptionList.keys(): 
			self._tpname = OptionList['tpname']
