# -*- coding: latin-1 -*-
'''
sapinfo.py

Copyright 2007 Mariano Nuez Di Croce @ CYBSEC

This file is part of sapyto, http://www.cybsec.com/EN/research/tools/sapyto.php

sapyto is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation version 2 of the License.

sapyto is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with sapyto; if not, write to the Free Software
Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
'''
import core.sapytoCore as core
from core.basePlugins.basePlugin import basePlugin
import core.kb.knowledgeBase as kb
import core.sapytoException as sapytoException
import core.output.outputManager as om

class sapinfo(basePlugin):
	
	def __init__(self):
		basePlugin.__init__( self )
		self._targets = {}
		
	def run(self):
		om.out.information('Starting \'sapinfo\' audit plugin.')
		om.out.information('Checking Target Information: ', newLine=False)
		if core.sCore.validateTargets(self._targets):
			om.out.information('OK')
		else:
			om.out.information('Not in plugin \'targets\' list. Plugin may fail.')
	
		data = []
		try:
			self._checkConnection()
			data =  self._conn.sapinfo()
		except sapytoException:
			raise
		om.out.information('Remote System Information:')
		for key in data.keys():
			data[ key ] = data[ key ].strip()
		om.out.information(self._formatOutput(data))
		kb.kb.save( 'sapinfo', 'systeminfo', data )
		om.out.information('Plugin \'sapinfo\' finished.')

	def _formatOutput(self, data):
		'''
		Formats output for sapinfo plugin.
		'''
		res = {}
		res['RFC Destination'] = data['RFCDEST']
		res['System ID'] = data['RFCSYSID']
		res['Hostame'] = data['RFCHOST2']
		res['IP Address'] = data['RFCIPADDR']
		res['Kernel Release'] = data['RFCKERNRL']
		res['Operating System'] = data['RFCOPSYS']
		res['Release Status of SAP System'] = data['RFCSAPRL']
		res['Central Database System'] = data['RFCDBSYS']
		res['Database Host'] = data['RFCDBHOST']
		res['Machine ID'] = data['RFCMACH']
		res['Character Set'] = data['RFCCHARTYP']
		res['Timezone'] = data['RFCTZONE'] + ' (diff from UTC in seconds)'
		res['Dayligth Saving Time'] = data['RFCDAYST']
		if data['RFCINTTYP']  == 'LIT':
			res['Integer Format'] = 'Little Endian'
		else:
			res['Integer Format'] = 'Big Endian'
		if data['RFCFLOTYP']  == 'IE3':
			res['Float Type Format'] = 'IEEE'
		else:
			res['Float Type Format'] = 'IBM/370'
		res['RFC Log Version'] = data['RFCPROTO']
		
		ret = ''
		for key in res.keys():
			ret += '\t' + key + ': ' + res[key] + '\n' 
		return ret
