# -*- coding: latin-1 -*-
'''
attack.py

Copyright 2007 Mariano Nuez Di Croce @ CYBSEC

This file is part of sapyto, http://www.cybsec.com/EN/research/tools/sapyto.php

sapyto is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation version 2 of the License.

sapyto is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with sapyto; if not, write to the Free Software
Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
'''
import core.sapytoCore
import core.output.outputManager as om
from core.sapytoException import sapytoException
import core.kb.knowledgeBase as kb
from core.ui.consoleUi.pluginConfig import pluginConfig
from core.ui.consoleUi.consoleMenu import consoleMenu


class attack(consoleMenu):
	'''
	This is the attack configuration menu for the console.
	
	@author: Andres Riancho ( andres.riancho@gmail.com )	
	'''
	def __init__( self, sapyto, commands=[] ):
		consoleMenu.__init__(self)
		self._menu = {'help':self._help, 'fastattack':self._fastattack, 'back':self._back}
		self._sapyto = sapyto
		self._commands = commands
	
	def sh( self ):
		'''
		Starts the shell's main loop.
		
		@return: The prompt
		'''
		prompt = 'sapyto/attack>>> '
		self._mainloop( prompt )
		
	def _exec( self, command ):
		'''
		Executes a user input.
		'''
		command, parameters = self._parse( command )

		if command in self._menu.keys():
			func = self._menu[command]
			try:
				res = func(parameters)
			except KeyboardInterrupt,k:
				raise k
			return res
		else:
			om.out.console( 'command not found.' )
			return True
		
	def _help( self, parameters ):
		'''
		Prints a help message to the user.
		'''
		if len( parameters ) == 0:
			self.mprint('The following commands are available:\n','')
			self.mprint('help','You are here. help [command] prints more specific help.')
			self.mprint('fastattack','launchs a individual attack plugin.')
			self.mprint('back','Return to previous menu.')			

		else:
			if len( parameters ) == 1:
				if parameters[0] in self._menu.keys():
					if parameters[0] == 'attack':
						om.out.console( 'exploit a vulnerability found by audit plugins.')
						om.out.console( 'Sintax: attack {plugin name}')
					elif parameters[0] == 'fastattack':
						om.out.console( 'attack a vulnerability based on external parameters.')
						om.out.console( 'Sintax: fastattack {plugin name}')
						om.out.console( 'Example: fastattack rfcexec')
					elif parameters[0] == 'list':
						om.out.console( 'List all available attack plugins.')
						om.out.console( 'Sintax: list' )
		
		return True
	
	def _fastattack( self , parameters):
		'''
		launch an attack plugin
		'''
		if not len( parameters ):
			self._help( ['attack'] )
		else:
			pluginName = parameters[0]
			if pluginName not in self._sapyto.getPluginList('attack'):
				om.out.console( 'Unknown plugin. Go to plugin menu to view available plugins.' )
			else:
				plugin = self._sapyto.getPluginInstance( pluginName )
				plugin.setConn(self._sapyto.getConn())
				#plugin.setCore(self._sapyto)
				plugin.setCommands(self._commands)
				plugin.run()				
			return True
	
	def _back( self, parameters ):
		return False
