# -*- coding: latin-1 -*-
'''
consoleUi.py

Copyright 2007 Mariano Nuez Di Croce @ CYBSEC

This file is part of sapyto, http://www.cybsec.com/EN/research/tools/sapyto.php

sapyto is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation version 2 of the License.

sapyto is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with sapyto; if not, write to the Free Software
Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
'''
import sys
try:
	# Some traditional imports
	import traceback
	from random import randint
	
	# Import all other menu's
	import core.ui.consoleUi.plugins as plugins
	import core.ui.consoleUi.targets as targets
	import core.ui.consoleUi.attack as attack
	
	# Import main
	from core.ui.consoleUi.consoleMenu import consoleMenu
	import core.output.outputManager as om	
	from core.sapytoException import  sapytoException
	import core.sapytoCore as core
	
except KeyboardInterrupt:
	print 'Exiting before importing modules.'
	sys.exit(0)
	
class consoleUi(consoleMenu):
	'''
	A console user interface.
	
	@author: Andres Riancho ( andres.riancho@gmail.com )	
	'''
	def __init__( self, commands=[] ):
		consoleMenu.__init__(self)
		self._menu = {'help':self._rootHelp, 'targets':self._target, 'plugins':self._plugins, 'start':self.start, 'attack':self._attack, 'exit':self._exit,}
		self._commands = commands
		# I will use pop(), so I need a reversed list.
		self._commands.reverse()
		self._targets = []
		
		self._sapyto = core.sCore
	
	def sh( self ):
		'''
		Starts the shell's main loop.
		
		@parameter commands: A list of commands to run.
		@return: The prompt
		'''
		prompt = 'sapyto>>> '
		self._mainloop( prompt )
		
	def _exec( self, command ):
		'''
		Executes a user input.
		'''
		command, parameters = self._parse( command )

		if command in self._menu.keys():
			func = self._menu[command]
			return func(parameters)
		else:
			om.out.console( 'command not found' )
		
	def _rootHelp( self, parameters ):
		'''
		Prints a help message to the user.
		'''
		if len(parameters) == 0 :
			self.mprint('The following commands are available:','')
			self.mprint('help','You are here. help [command] prints more specific help.')
			self.mprint('plugins','Enable, disable and configure plugins.')
			self.mprint('exploit','Start exploiting.')
			self.mprint('targets','Set the targets information.')
			self.mprint('exit','Exit sapyto.')
		elif parameters[0] == 'targets':
			self.mprint('Enter targets configuration.','')
		elif parameters[0] == 'exploit':
			self.mprint('Enter the exploit configuration.','')
		elif parameters[0] == 'plugins':
			self.mprint('Enter the plugin configuration.','')
			
	def _plugins( self, parameters  ):
		'''
		Opens a plugins config menu
		'''
		p = plugins.plugins( self._sapyto, self._commands  )
		try:
			p.sh()
		except KeyboardInterrupt,k:
			om.out.console('')
	
	def _attack( self, parameters  ):
		'''
		Opens an attack config menu
		'''
		e = attack.attack( self._sapyto, self._commands  )
		try:
			e.sh()
		except KeyboardInterrupt,k:
			om.out.console('')
	
	
	def _target( self, parameters ):
		'''
		Sets the target system information
		'''
		p = targets.targets( self._sapyto, self._commands  )
		try:
			p.sh()
		except KeyboardInterrupt,k:
			om.out.console('')
		
		
	def start( self, parameters  ):
		'''
		Starts the work.
		'''
		try:
			self._sapyto.start()
		except sapytoException, e:
			om.out.console( str(e) )
		except KeyboardInterrupt, e:
			self._exit()
		
		return True
		
	def _getRndExitMsg( self ):
		'''
		@return: A random exit msg.
		'''
		res = []
		res.append('bye.')
		
		#res = res[ randint( 0, len(res) -1 ) ]
		
		return res[0]
	
	
	def _exit( self, parameters = [] ):
		om.out.console( '' )
		om.out.console( self._getRndExitMsg() )
		sys.exit(0)
	
	_back = _exit
